__all__ = [
    "BaseModel",
    "Field",
    "hf",
    "logger",
    "app",
    "legacy_agents",
    "cli",
    "chainofthought",
    "classify",
    "completion",
    "embedding",
    "image",
    "code",
    "delegate",
    "extract",
    "function",
    "generate",
    "qdrant",
    "sql",
    "transcribe",
    "speak",
    "zyxModuleLoader",
]

# --- zyx ----------------------------------------------------------------

from .core.ext import BaseModel, Field, zyxModuleLoader


class legacy_agents(zyxModuleLoader):
    pass


legacy_agents.init("zyx.client.agents", "Agents")


class app(zyxModuleLoader):
    pass


app.init("zyx.client.app", "app")


class cli(zyxModuleLoader):
    pass


cli.init("zyx.client.app", "cli")


class chainofthought(zyxModuleLoader):
    pass


chainofthought.init("zyx.client.fn", "chainofthought")


class classify(zyxModuleLoader):
    pass


classify.init("zyx.client.fn", "classify")


class code(zyxModuleLoader):
    pass


code.init("zyx.client.fn", "code")


class extract(zyxModuleLoader):
    pass


extract.init("zyx.client.fn", "extract")


class delegate(zyxModuleLoader):
    pass


delegate.init("zyx.client.fn", "delegate")


class function(zyxModuleLoader):
    pass


function.init("zyx.client.fn", "function")


class image(zyxModuleLoader):
    pass


image.init("zyx.multimodal.main", "image")


class generate(zyxModuleLoader):
    pass


generate.init("zyx.client.fn", "generate")


class completion(zyxModuleLoader):
    pass


completion.init("zyx.client.main", "completion")


class hf(zyxModuleLoader):
    pass


hf.init("huggingface_hub.inference._client", "InferenceClient")


class logger(zyxModuleLoader):
    pass


logger.init("loguru", "logger")


class qdrant(zyxModuleLoader):
    pass


qdrant.init("zyx.data.qdr", "Qdrant")


class embedding(zyxModuleLoader):
    pass


embedding.init("litellm.main", "embedding")


class speak(zyxModuleLoader):
    pass


speak.init("zyx.multimodal.main", "openai_text_to_speech")


class transcribe(zyxModuleLoader):
    pass


transcribe.init("zyx.multimodal.main", "openai_speech_to_text")


class sql(zyxModuleLoader):
    pass


sql.init("zyx.data.main", "SQLModelVectorDB")


def chat():
    import argparse
    from .client.app import cli as chat_cli

    parser = argparse.ArgumentParser(description="ZYX Chat CLI")

    parser.add_argument(
        "--provider", type=str, help="The provider instance to handle the chat logic"
    )
    parser.add_argument(
        "--messages", type=str, help="The messages to send to the model"
    )
    parser.add_argument(
        "--model",
        type=str,
        default="gpt-4o-mini",
        help="The model to use for completions",
    )
    parser.add_argument("--tools", nargs="+", help="The tools to use for completions")
    parser.add_argument(
        "--run-tools", type=bool, default=True, help="Whether to run the tools"
    )
    parser.add_argument(
        "--response-model",
        type=str,
        help="The Pydantic response model to use for completions",
    )
    parser.add_argument(
        "--mode", type=str, default="tools", help="The mode to use for completions"
    )
    parser.add_argument("--base-url", type=str, help="The base URL for the API")
    parser.add_argument("--api-key", type=str, help="The API key to use for the API")
    parser.add_argument(
        "--organization", type=str, help="The organization to use for the API"
    )
    parser.add_argument("--top-p", type=float, help="The top-p value for completions")
    parser.add_argument(
        "--temperature", type=float, help="The temperature value for completions"
    )
    parser.add_argument(
        "--max-tokens", type=int, help="The maximum number of tokens for completions"
    )
    parser.add_argument(
        "--max-retries",
        type=int,
        default=3,
        help="The maximum number of retries for completions",
    )
    parser.add_argument(
        "--verbose", type=bool, default=False, help="Whether to print verbose output"
    )
    parser.add_argument(
        "--background",
        type=str,
        default="midnight_black",
        help="Background color of the app",
    )
    parser.add_argument(
        "--text", type=str, default="steel_grey", help="Text color in the chat display"
    )
    parser.add_argument(
        "--input-field", type=str, default="ocean_blue", help="Color of the input field"
    )

    args = parser.parse_args()

    # Convert args to a dictionary and remove None values
    chat_args = {k: v for k, v in vars(args).items() if v is not None}

    chat_cli(**chat_args)



from crewai_tools.tools.base_tool import BaseTool
from langchain_core.tools.convert import tool

from . import tools as tools
from ..core.ext import zyxModuleLoader

class agents(zyxModuleLoader):
    pass
agents.init("zyx.agents.main", "Agents")




__all__ = [
    'BrowserbaseLoadTool',
    'CodeDocsSearchTool',
    'CodeInterpreterTool',
    'ComposioTool',
    'CSVSearchTool',
    'DallETool',
    'DirectoryReadTool',
    'DirectorySearchTool',
    'DOCXSearchTool',
    'FileReadToolSchema',
    'FileWriterTool',
    'FileWriterToolInput',
    'GithubSearchTool',
    'MySQLSearchTool',
    'PDFTextWritingTool',
    'ScrapeElementFromWebsiteTool',
    'SerperDevTool',
    'TXTSearchTool',
    'VisionTool',
    'WebsiteSearchTool',
    'YoutubeChannelSearchTool',
    'YoutubeVideoSearchTool'
]

# Base Tools
from crewai_tools.tools.browserbase_load_tool.browserbase_load_tool import BrowserbaseLoadTool
from crewai_tools.tools.code_docs_search_tool.code_docs_search_tool import CodeDocsSearchTool
from crewai_tools.tools.code_interpreter_tool.code_interpreter_tool import CodeInterpreterTool
from crewai_tools.tools.composio_tool.composio_tool import ComposioTool
from crewai_tools.tools.csv_search_tool.csv_search_tool import CSVSearchTool
from crewai_tools.tools.dalle_tool.dalle_tool import DallETool
from crewai_tools.tools.directory_read_tool.directory_read_tool import DirectoryReadTool
from crewai_tools.tools.directory_search_tool.directory_search_tool import DirectorySearchTool
from crewai_tools.tools.docx_search_tool.docx_search_tool import DOCXSearchTool
from crewai_tools.tools.file_read_tool.file_read_tool import FileReadToolSchema
from crewai_tools.tools.file_writer_tool.file_writer_tool import FileWriterTool, FileWriterToolInput
from crewai_tools.tools.github_search_tool.github_search_tool import GithubSearchTool
from crewai_tools.tools.mysql_search_tool.mysql_search_tool import MySQLSearchTool
from crewai_tools.tools.pdf_text_writing_tool.pdf_text_writing_tool import PDFTextWritingTool
from crewai_tools.tools.scrape_element_from_website.scrape_element_from_website import ScrapeElementFromWebsiteTool
from crewai_tools.tools.serper_dev_tool.serper_dev_tool import SerperDevTool
from crewai_tools.tools.txt_search_tool.txt_search_tool import TXTSearchTool
from crewai_tools.tools.vision_tool.vision_tool import VisionTool
from crewai_tools.tools.website_search.website_search_tool import WebsiteSearchTool
from crewai_tools.tools.youtube_channel_search_tool.youtube_channel_search_tool import YoutubeChannelSearchTool
from crewai_tools.tools.youtube_video_search_tool.youtube_video_search_tool import YoutubeVideoSearchTool


from qdrant_client.http import models as rest
from pydantic import BaseModel, Field
from typing import Callable, List, Union, Optional, Literal
import uuid
import logging

from ..types import ClientModeParams, ClientParams
from .. import logger


class QdrantNode(BaseModel):
    id: str
    text: str
    embedding: List[float]
    metadata: Optional[dict] = None


class SearchResponse(BaseModel):
    query: str
    results: List[QdrantNode] = Field(default_factory=list)


class Qdrant:
    def __init__(
        self,
        collection_name: str = "my_collection",
        vector_size: int = 1536,
        distance: Literal["Cosine", "Euclid", "Dot"] = "Cosine",
        location: str = ":memory:",
        host: str = None,
        port: int = 6333,
        embedding_model: str = "text-embedding-3-small",
        embedding_dimensions: Optional[int] = None,
        embedding_api_key: Optional[str] = None,
        embedding_api_base: Optional[str] = None,
        embedding_api_version: Optional[str] = None,
    ):
        from qdrant_client.http.models import Distance
        from qdrant_client import QdrantClient

        self.client = QdrantClient(location=location, host=host, port=port)
        self.collection_name = collection_name
        self.vector_size = vector_size
        self.distance = Distance(distance)
        self.embedding_model = embedding_model
        self.embedding_dimensions = embedding_dimensions
        self.embedding_api_key = embedding_api_key
        self.embedding_api_base = embedding_api_base
        self.embedding_api_version = embedding_api_version

        self._create_collection()

    def _create_collection(self):
        from qdrant_client.http.models import VectorParams

        try:
            self.client.get_collection(self.collection_name)
            logger.info(f"Collection '{self.collection_name}' already exists.")
        except Exception as e:
            logger.info(
                f"Collection '{self.collection_name}' does not exist. Creating it now."
            )
            self.client.create_collection(
                collection_name=self.collection_name,
                vectors_config=VectorParams(
                    size=self.vector_size, distance=self.distance
                ),
            )
            logger.info(f"Collection '{self.collection_name}' created successfully.")

    def _get_embedding(self, text: str) -> List[float]:
        from litellm.main import embedding as litellm_embedding

        try:
            response = litellm_embedding(
                model=self.embedding_model,
                input=[text],
                dimensions=self.embedding_dimensions,
                api_key=self.embedding_api_key,
                api_base=self.embedding_api_base,
                api_version=self.embedding_api_version,
            )

            embedding_data = response.get("data", None)
            if (
                embedding_data
                and isinstance(embedding_data, list)
                and len(embedding_data) > 0
            ):
                embedding_vector = embedding_data[0].get("embedding", None)
                if isinstance(embedding_vector, list) and all(
                    isinstance(x, float) for x in embedding_vector
                ):
                    return embedding_vector
                else:
                    raise ValueError(
                        "Invalid embedding format: Expected a list of floats within the 'embedding' key"
                    )
            else:
                raise ValueError("Embedding data is missing or improperly formatted")
        except Exception as e:
            logger.error(f"Error generating embedding: {e}")
            raise

    def add(self, texts: Union[str, List[str]], metadata: Optional[dict] = None):
        from qdrant_client.http.models import PointStruct

        if isinstance(texts, str):
            texts = [texts]

        points = []
        for text in texts:
            try:
                embedding_vector = self._get_embedding(text)
                node_id = str(uuid.uuid4())
                point = PointStruct(
                    id=str(uuid.uuid4()),
                    vector=embedding_vector,
                    payload={"text": text, "metadata": metadata or {}},
                )
                points.append(point)
            except Exception as e:
                logger.error(f"Error processing text: {text}. Error: {e}")

        if points:
            try:
                self.client.upsert(collection_name=self.collection_name, points=[point])
                logger.info(
                    f"Successfully added {len(points)} points to the collection."
                )
            except Exception as e:
                logger.error(f"Error upserting points to collection: {e}")
        else:
            logger.warning("No valid points to add to the collection.")

    def search(self, query: str, top_k: int = 5) -> SearchResponse:
        try:
            query_vector = self._get_embedding(query)
            search_results = self.client.search(
                collection_name=self.collection_name,
                query_vector=query_vector,
                limit=top_k,
            )

            nodes = []
            for result in search_results:
                payload = result.payload
                node = QdrantNode(
                    id=str(result.id),
                    text=payload.get("text", ""),
                    embedding=query_vector,
                    metadata=payload.get("metadata", {}),
                )
                nodes.append(node)
            return SearchResponse(query=query, results=nodes)
        except Exception as e:
            logger.error(f"Error during search: {e}")
            raise

    def completion(
        self,
        messages: Union[str, list[dict[str, str]]],
        model: Optional[str] = "gpt-4o-mini",
        top_k: Optional[int] = 5,
        tools: Optional[List[Union[Callable, dict, BaseModel]]] = None,
        run_tools: Optional[bool] = True,
        response_model: Optional[BaseModel] = None,
        mode: Optional[ClientModeParams] = "tools",
        base_url: Optional[str] = None,
        api_key: Optional[str] = None,
        organization: Optional[str] = None,
        top_p: Optional[float] = None,
        temperature: Optional[float] = None,
        max_tokens: Optional[int] = None,
        max_retries: Optional[int] = 3,
        verbose: Optional[bool] = False,
    ):
        if isinstance(messages, str):
            query = messages
            messages = [{"role": "user", "content": messages}]
        elif isinstance(messages, list):
            query = messages[-1].get("content", "")

        try:
            results = self.search(query, top_k=top_k)
        except Exception as e:
            logger.error(f"Error during search: {e}")
            results = SearchResponse(query=query)

        if messages and isinstance(messages, list):
            if not any(message.get("role", "") == "system" for message in messages):
                messages.insert(
                    0,
                    {
                        "role": "system",
                        "content": f"You have retrieved the following relevant information. Use only if relevant {results}",
                    },
                )
            else:
                for message in messages:
                    if message.get("role", "") == "system":
                        message["content"] += (
                            f" You have retrieved the following relevant information. Use only if relevant {results}"
                        )
                        break
        from ..client.main import Client

        return Client().completion(
            messages=messages,
            model=model,
            tools=tools,
            run_tools=run_tools,
            response_model=response_model,
            mode=mode,
            base_url=base_url,
            api_key=api_key,
            organization=organization,
            top_p=top_p,
            temperature=temperature,
            max_tokens=max_tokens,
            max_retries=max_retries,
            verbose=verbose,
        )


if __name__ == "__main__":
    try:
        qdrant = Qdrant(collection_name="my_collection", vector_size=1536)
        qdrant.add(["Hello, world!", "How are you?", "What's up?"])
        results = qdrant.search("How are you?")
        for result in results.results:
            print(f"ID: {result.id}, Text: {result.text}, Metadata: {result.metadata}")
    except Exception as e:
        logger.error(f"Error in main execution: {e}")




__all__ = [
    'BrowserbaseLoadTool',
    'CodeDocsSearchTool',
    'CodeInterpreterTool',
    'ComposioTool',
    'CSVSearchTool',
    'DallETool',
    'DirectoryReadTool',
    'DirectorySearchTool',
    'DOCXSearchTool',
    'FileReadToolSchema',
    'FileWriterTool',
    'FileWriterToolInput',
    'GithubSearchTool',
    'MySQLSearchTool',
    'PDFTextWritingTool',
    'ScrapeElementFromWebsiteTool',
    'SerperDevTool',
    'TXTSearchTool',
    'VisionTool',
    'WebsiteSearchTool',
    'YoutubeChannelSearchTool',
    'YoutubeVideoSearchTool'
]

from ..core.ext import zyxModuleLoader

class BrowserbaseLoadTool(zyxModuleLoader):
    pass
BrowserbaseLoadTool.init("crewai_tools.tools.browserbase_load_tool.browserbase_load_tool", "BrowserbaseLoadTool")

class CodeDocsSearchTool(zyxModuleLoader):
    pass
CodeDocsSearchTool.init("crewai_tools.tools.code_docs_search_tool.code_docs_search_tool", "CodeDocsSearchTool")

class CodeInterpreterTool(zyxModuleLoader):
    pass
CodeInterpreterTool.init("crewai_tools.tools.code_interpreter_tool.code_interpreter_tool", "CodeInterpreterTool")

class ComposioTool(zyxModuleLoader):
    pass
ComposioTool.init("crewai_tools.tools.composio_tool.composio_tool", "ComposioTool")

class CSVSearchTool(zyxModuleLoader):
    pass
CSVSearchTool.init("crewai_tools.tools.csv_search_tool.csv_search_tool", "CSVSearchTool")

class DallETool(zyxModuleLoader):
    pass
DallETool.init("crewai_tools.tools.dalle_tool.dalle_tool", "DallETool")

class DirectoryReadTool(zyxModuleLoader):
    pass
DirectoryReadTool.init("crewai_tools.tools.directory_read_tool.directory_read_tool", "DirectoryReadTool")

class DirectorySearchTool(zyxModuleLoader):
    pass
DirectorySearchTool.init("crewai_tools.tools.directory_search_tool.directory_search_tool", "DirectorySearchTool")

class DOCXSearchTool(zyxModuleLoader):
    pass
DOCXSearchTool.init("crewai_tools.tools.docx_search_tool.docx_search_tool", "DOCXSearchTool")

class FileReadToolSchema(zyxModuleLoader):
    pass
FileReadToolSchema.init("crewai_tools.tools.file_read_tool.file_read_tool", "FileReadToolSchema")

class FileWriterTool(zyxModuleLoader):
    pass
FileWriterTool.init("crewai_tools.tools.file_writer_tool.file_writer_tool", "FileWriterTool")

class FileWriterToolInput(zyxModuleLoader):
    pass
FileWriterToolInput.init("crewai_tools.tools.file_writer_tool.file_writer_tool", "FileWriterToolInput")

class GithubSearchTool(zyxModuleLoader):
    pass
GithubSearchTool.init("crewai_tools.tools.github_search_tool.github_search_tool", "GithubSearchTool")

class MySQLSearchTool(zyxModuleLoader):
    pass
MySQLSearchTool.init("crewai_tools.tools.mysql_search_tool.mysql_search_tool", "MySQLSearchTool")

class PDFTextWritingTool(zyxModuleLoader):
    pass
PDFTextWritingTool.init("crewai_tools.tools.pdf_text_writing_tool.pdf_text_writing_tool", "PDFTextWritingTool")

class ScrapeElementFromWebsiteTool(zyxModuleLoader):
    pass
ScrapeElementFromWebsiteTool.init("crewai_tools.tools.scrape_element_from_website.scrape_element_from_website", "ScrapeElementFromWebsiteTool")

class SerperDevTool(zyxModuleLoader):
    pass
SerperDevTool.init("crewai_tools.tools.serper_dev_tool.serper_dev_tool", "SerperDevTool")

class TXTSearchTool(zyxModuleLoader):
    pass
TXTSearchTool.init("crewai_tools.tools.txt_search_tool.txt_search_tool", "TXTSearchTool")

class VisionTool(zyxModuleLoader):
    pass
VisionTool.init("crewai_tools.tools.vision_tool.vision_tool", "VisionTool")

class WebsiteSearchTool(zyxModuleLoader):
    pass
WebsiteSearchTool.init("crewai_tools.tools.website_search.website_search_tool", "WebsiteSearchTool")

class YoutubeChannelSearchTool(zyxModuleLoader):
    pass
YoutubeChannelSearchTool.init("crewai_tools.tools.youtube_channel_search_tool.youtube_channel_search_tool", "YoutubeChannelSearchTool")

class YoutubeVideoSearchTool(zyxModuleLoader):
    pass
YoutubeVideoSearchTool.init("crewai_tools.tools.youtube_video_search_tool.youtube_video_search_tool", "YoutubeVideoSearchTool")
